#!/bin/ash
#### Description: Decide on basis of captured packets whether the router is busy or idle
#### Used captured packets are saved in pcap file generated from the router's tracing tool
#### Router: Vodafone EasyBox 804 04.03
#### Firmware-Version: CIS804-04.03
CWD=$(dirname $0)

while getopts r:p:du opt
do
	case $opt in
		r) ROUTER_IP=$OPTARG;;
		p) PASSWORD=$OPTARG;;
		d) OPTION="download";;
		u) OPTION="upload";;
	esac
done

URL_PREFIX="http://$ROUTER_IP/main.cgi"


get_cookies(){
	url="$URL_PREFIX?page=login.html"
	DM_COOKIE=$(curl -s -m 5 -c $CWD/cookie.txt $url | grep dm_cookie | awk -F"dm_cookie='" '{print $2}' | awk -F"'" '{print $1}')
	WBM_COOKIE_SESSION=$(cat $CWD/cookie.txt | grep wbm_cookie_session_id | awk '{print $7}')
	cat $CWD/cookie.txt | sed "s/${ROUTER_IP}//g" | sed "s:\t\/\t:\t\t:g " > $CWD/cookie2.txt
}


get_auth_key(){
	url="$URL_PREFIX?js=rg_config.js"
	curl -s -m 5 -b $CWD/cookie.txt -o $CWD/config.js $url > /dev/null
	AUTH_KEY=$(cat $CWD/config.js | grep auth_key | grep -oE "[0-9]+")
}


encrypt_md5_password(){
	PASSWORD_MD5=$(echo -n "$PASSWORD$AUTH_KEY" | md5sum | awk '{print $1}')
	#echo "PASSWORD: $PASSWORD"
	#echo "AUTH_KEY: $AUTH_KEY"
	#echo "PASSWORD_MD5: $PASSWORD_MD5"
}

login(){
	url="http://$ROUTER_IP/data_model.cgi"
	get_cookies
	get_auth_key
	encrypt_md5_password
	echo -e "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\"><soapenv:Header><DMCookie>${DM_COOKIE}</DMCookie></soapenv:Header><soapenv:Body><cwmp:Login xmlns=\"\"><ParameterList><Username>vodafone</Username><Password>${PASSWORD_MD5}</Password><AllowRelogin>0</AllowRelogin></ParameterList></cwmp:Login></soapenv:Body></soapenv:Envelope>" > $CWD/step1_soap_data.xml
	curl -s $url -b "$CWD/cookie2.txt" -c $CWD/step2_cookies.txt -X POST -H 'Accept: application/xml, text/xml, */*; q=0.01' -H 'Accept-Encoding: gzip, deflate' -H 'Content-Type: text/xml; charset="utf-8"' -H 'Connection: keep-alive' -H 'Cache-Control: no-cache' -H "Cookie: wbm_cookie_session_id=${WBM_COOKIE_SESSION}" -H 'SOAPAction: cwmp:Login' -H 'X-Requested-With: XMLHttpRequest' \
    --data @$CWD/step1_soap_data.xml -o $CWD/step2_answer.html
	WBM_COOKIE_SESSION2=$(cat $CWD/step2_cookies.txt | grep "wbm_cookie_session_id" | awk '{print $7}')
	curl -s "$URL_PREFIX/main.cgi?page=app.html" -b $CWD/step2_cookies.txt -o $CWD/step3_answer.html
	DM_COOKIE2=$(cat $CWD/step3_answer.html | grep dm_cookie | awk -F"dm_cookie='" '{print $2}' | awk -F"'" '{print $1}')
}


statistics_dsl(){
	### DSL total statistics
	echo "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\"><soapenv:Header><DMCookie>${DM_COOKIE2}</DMCookie><SessionNotRefresh>1</SessionNotRefresh></soapenv:Header><soapenv:Body><cwmp:GetParameterValues xmlns=\"\"><ParameterNames><string>InternetGatewayDevice.WANDevice.3.WANDSLInterfaceConfig.Stats.Total.TransmitBlocks</string><string>InternetGatewayDevice.WANDevice.3.WANDSLInterfaceConfig.Stats.Total.ReceiveBlocks</string><string>InternetGatewayDevice.WANDevice.3.WANDSLInterfaceConfig.UpstreamCurrRate</string><string>InternetGatewayDevice.WANDevice.3.WANDSLInterfaceConfig.DownstreamCurrRate</string><string>InternetGatewayDevice.WANDevice.3.WANDSLInterfaceConfig.UpstreamMaxRate</string><string>InternetGatewayDevice.WANDevice.3.WANDSLInterfaceConfig.DownstreamMaxRate</string></ParameterNames></cwmp:GetParameterValues></soapenv:Body></soapenv:Envelope>" > $CWD/step4_soap_data.xml
	dsl_statistics=$(curl -s "http://${ROUTER_IP}/data_model.cgi" \
		-b cookie2.txt \
		-X POST \
		-H 'Accept: application/xml, text/xml, */*; q=0.01' \
		-H 'Accept-Encoding: gzip, deflate' \
		-H 'Accept-Language: de,en-US;q=0.7,en;q=0.3' \
		-H 'Connection: keep-alive' \
		-H 'Content-Type: text/xml; charset="utf-8"' \
		-H "Cookie: wbm_cookie_session_id=${WBM_COOKIE_SESSION2}" \
		-H "Host: ${ROUTER_IP}" \
		-H 'Method: POST' \
		-H 'Pragma: no-cache' \
		-H 'Referer: http://${ROUTER_IP}/main.cgi?page=app.html' \
		-H 'SOAPAction: cwmp:GetParameterValues' \
		-H 'User-Agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.94 Safari/537.36' \
		-H 'X-Requested-With: XMLHttpRequest' \
		--data @$CWD/step4_soap_data.xml)
	wandsl_blocks_tx=$(echo $dsl_statistics | awk -F'Total.TransmitBlocks</Name> <Value xsi:type="xsd:unsignedInt">' '{print $2}'| awk -F'</Value>' '{print $1}')
	wandsl_blocks_rx=$(echo $dsl_statistics | awk -F'Total.ReceiveBlocks</Name> <Value xsi:type="xsd:unsignedInt">' '{print $2}'| awk -F'</Value>' '{print $1}')
	wandsl_currate_up=$(echo $dsl_statistics | awk -F'UpstreamCurrRate</Name> <Value xsi:type="xsd:unsignedInt">' '{print $2}' | awk -F'</Value>' '{print $1}')
	wandsl_currate_down=$(echo $dsl_statistics | awk -F'DownstreamCurrRate</Name> <Value xsi:type="xsd:unsignedInt">' '{print $2}' | awk -F'</Value>' '{print $1}')
	wandsl_maxrate_up=$(echo $dsl_statistics | awk -F'UpstreamMaxRate</Name> <Value xsi:type="xsd:unsignedInt">' '{print $2}' | awk -F'</Value>' '{print $1}')
	wandsl_maxrate_down=$(echo $dsl_statistics | awk -F'DownstreamMaxRate</Name> <Value xsi:type="xsd:unsignedInt">' '{print $2}' | awk -F'</Value>' '{print $1}')
	echo "Total Transmit Blocks:               $wandsl_blocks_tx"
	echo "Total Receive Blocks:                $wandsl_blocks_rx"
	echo "Current DSL Upstream Rate:           $wandsl_currate_up"
	echo "Current DSL Downstream Rate:         $wandsl_currate_down"
	echo "Maximal DSL Upstream Rate:           $wandsl_maxrate_up"
	echo "Maximal DSL Downstream Rate:         $wandsl_maxrate_down"

}


statistics_wan(){
	### WAN interface properties common across all connection instances
	echo "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\"><soapenv:Header><DMCookie>${DM_COOKIE2}</DMCookie><SessionNotRefresh>1</SessionNotRefresh></soapenv:Header><soapenv:Body><cwmp:GetParameterValues><ParameterNames><string>InternetGatewayDevice.WANDevice.3.WANCommonInterfaceConfig.TotalBytesSent</string><string>InternetGatewayDevice.WANDevice.3.WANCommonInterfaceConfig.TotalBytesReceived</string><string>InternetGatewayDevice.WANDevice.3.WANCommonInterfaceConfig.TotalPacketsSent</string><string>InternetGatewayDevice.WANDevice.6.WANCommonInterfaceConfig.TotalPacketsReceived</string><string>InternetGatewayDevice.WANDevice.3.WANCommonInterfaceConfig.Layer1UpstreamMaxBitRate</string><string>InternetGatewayDevice.WANDevice.3.WANCommonInterfaceConfig.Layer1DownstreamMaxBitRate</string></ParameterNames></cwmp:GetParameterValues></soapenv:Body></soapenv:Envelope>" > $CWD/step4_soap_data.xml
	wan_statistics=$(curl -s "http://${ROUTER_IP}/data_model.cgi" \
		-b cookie2.txt \
		-X POST \
		-H 'Accept: application/xml, text/xml, */*; q=0.01' \
		-H 'Accept-Encoding: gzip, deflate' \
		-H 'Accept-Language: de,en-US;q=0.7,en;q=0.3' \
		-H 'Connection: keep-alive' \
		-H 'Content-Type: text/xml; charset="utf-8"' \
		-H "Cookie: wbm_cookie_session_id=${WBM_COOKIE_SESSION2}" \
		-H "Host: ${ROUTER_IP}" \
		-H 'Method: POST' \
		-H 'Pragma: no-cache' \
		-H 'Referer: http://${ROUTER_IP}/main.cgi?page=app.html' \
		-H 'SOAPAction: cwmp:GetParameterValues' \
		-H 'X-Requested-With: XMLHttpRequest' \
		--data @$CWD/step4_soap_data.xml)
	wan_bytes_tx=$(echo $wan_statistics | awk -F'TotalBytesSent</Name> <Value xsi:type="xsd:unsignedInt">' '{print $2}' | awk -F'</Value>' '{print $1}')
	wan_bytes_rx=$(echo $wan_statistics | awk -F'TotalBytesReceived</Name> <Value xsi:type="xsd:unsignedInt">' '{print $2}' | awk -F'</Value>' '{print $1}')
	wan_packets_tx=$(echo $wan_statistics | awk -F'TotalPacketsSent</Name> <Value xsi:type="xsd:unsignedInt">' '{print $2}' | awk -F'</Value>' '{print $1}')
	wan_packets_rx=$(echo $wan_statistics | awk -F'TotalPacketsReceived</Name> <Value xsi:type="xsd:unsignedInt">' '{print $2}' | awk -F'</Value>' '{print $1}')
	wan_l1_maxbitrate_tx=$(echo $wan_statistics | awk -F'Layer1UpstreamMaxBitRate</Name> <Value xsi:type="xsd:unsignedInt">' '{print $2}' | awk -F'</Value>' '{print $1}')
	wan_l1_maxbitrate_rx=$(echo $wan_statistics | awk -F'Layer1DownstreamMaxBitRate</Name> <Value xsi:type="xsd:unsignedInt">' '{print $2}' | awk -F'</Value>' '{print $1}')
	#echo "Total WAN Bytes Sent:                $wan_bytes_tx"
	#echo "Techo "Total WAN Bytes Received:            $wan_bytes_rx"
	#echo "Techo "Total WAN Packets Sent:              $wan_packets_tx"
	#echo "Techo "Total WAN Packets Received:          $wan_packets_rx"
	#echo "Techo "Maximal Upstream Bit Rate for WAN:   $wan_l1_maxbitrate_tx"
	#echo "Techo "Maximal Downstream Bit Rate for WAN: $wan_l1_maxbitrate_rx"
}

statistics_lan(){
	# LAN interface statistics
	echo "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\"><soapenv:Header><DMCookie>${DM_COOKIE2}</DMCookie><SessionNotRefresh>1</SessionNotRefresh></soapenv:Header><soapenv:Body><cwmp:GetParameterValues><ParameterNames><string>InternetGatewayDevice.LANDevice.2.LANEthernetInterfaceConfig.1.Stats.BytesSent</string></ParameterNames></cwmp:GetParameterValues></soapenv:Body></soapenv:Envelope>" > $CWD/step4_soap_data.xml
	lan_statistics=$(curl -s "http://${ROUTER_IP}/data_model.cgi" \
		-b cookie2.txt \
		-X POST \
		-H 'Accept: application/xml, text/xml, */*; q=0.01' \
		-H 'Accept-Encoding: gzip, deflate' \
		-H 'Accept-Language: de,en-US;q=0.7,en;q=0.3' \
		-H 'Connection: keep-alive' \
		-H 'Content-Type: text/xml; charset="utf-8"' \
		-H "Cookie: wbm_cookie_session_id=${WBM_COOKIE_SESSION2}" \
		-H "Host: ${ROUTER_IP}" \
		-H 'Method: POST' \
		-H 'Pragma: no-cache' \
		-H 'Referer: http://${ROUTER_IP}/main.cgi?page=app.html' \
		-H 'SOAPAction: cwmp:GetParameterValues' \
		-H 'X-Requested-With: XMLHttpRequest' \
		--data @$CWD/step4_soap_data.xml)
	echo $lan_statistics
}

getRPCMethods(){
	echo "<soapenv:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\"><soapenv:Header><DMCookie>${DM_COOKIE2}</DMCookie><SessionNotRefresh>1</SessionNotRefresh></soapenv:Header><soapenv:Body><cwmp:GetRPCMethods/></soapenv:Body></soapenv:Envelope>" > $CWD/step4_soap_data.xml\
	curl "http://${ROUTER_IP}/data_model.cgi" \
		-b cookie2.txt \
		-X POST \
		-H 'Accept: application/xml, text/xml, */*; q=0.01' \
		-H 'Accept-Encoding: gzip, deflate' \
		-H 'Accept-Language: de,en-US;q=0.7,en;q=0.3' \
		-H 'Connection: keep-alive' \
		-H 'Content-Type: text/xml; charset="utf-8"' \
		-H "Cookie: wbm_cookie_session_id=${WBM_COOKIE_SESSION2}" \
		-H "Host: ${ROUTER_IP}" \
		-H 'Method: POST' \
		-H 'Pragma: no-cache' \
		-H 'Referer: http://${ROUTER_IP}/main.cgi?page=app.html' \
		-H 'X-Requested-With: XMLHttpRequest' \
                --data @$CWD/step4_soap_data.xml
}

get_total_frame_size_from_pcap(){
	totalLength=$(tcpdump -qens 0 host $ROUTER_IP -r packets.pcap | awk '{sum+=substr($7,1,length($7)-1)} END {print sum}')
}

calc_network_state(){
	statistics_wan
	through1=$(( wan_bytes_tx+wan_bytes_rx ))
	sleep $SLEEPTIME
	statistics_wan
	through2=$(( wan_bytes_tx+wan_bytes_rx ))
	through=$(( through2-through1 ))
	if [ "$through" -gt "$THRESHOLD" ]
	then
		exit $BUSY
	else
		exit $IDLE
	fi
}

login
#statistics_dsl
statistics_wan
#statistics_lan
#getRPCMethods
#calc_network_state

if [ "$OPTION" == "download" ]; then
	echo $wan_bytes_rx
elif [ "$OPTION" == "upload" ]; then
	echo $wan_bytes_tx
else
	echo "Download or upload parameter is missing."
fi

rm $CWD/config.js
rm $CWD/cookie.txt
rm $CWD/cookie2.txt
rm $CWD/step1_soap_data.xml
rm $CWD/step2_answer.html
rm $CWD/step2_cookies.txt
rm $CWD/step3_answer.html
rm $CWD/step4_soap_data.xml
