#!/bin/ash

# Tested for:
# -----------
# - Fritz!Box 7312, Fritz!OS 06.30, 1&1 edition
# - Fritz!Box 7330 SL, Fritz!OS 06.30, 1&1 edition
# - Fritz!Box 7360, Fritz!OS 06.34, M-net edition
# - Fritz!Box 7430, Fritz!OS 06.83
# - Fritz!Box 7560 LI, Fritz!OS 06.92
# - (DOES NOT WORK on) Fritz!Box 6490 Cable, Fritz!OS 06.87

# TODO: FIX bug that at first execution not all stats are shown
# TODO: \-> partially fixed, look for better solution than duplicating a request
# TODO: replace single [ and ] with double ones
# TODO: look for better method / string comparison with empty string

option1=$1
option2=$2

check_box_url(){
	_BOXURL=$(ifconfig | grep "inet " | awk '{print $2}' | grep -e '192.' -e '10.' -e '172.' | awk -F"." '{print $1"."$2"."$3"."1}')
}

create_xml(){
	xml_data="<?xml version='1.0' encoding='utf-8'?><s:Envelope s:encodingStyle='http://schemas.xmlsoap.org/soap/encoding/' xmlns:s='http://schemas.xmlsoap.org/soap/envelope/'><s:Body><u:$action xmlns:u='$uri'></u:$action></s:Body></s:Envelope>"
}

curl_request(){
	curl_output=$(curl -s -m 5 --anyauth -u "dslf-config:weber7445" http://192.168.178.1:49000$control_url -H "Content-Type: text/xml; charset='utf-8'" -H "SoapAction:$service#$action" -d "$xml_data")
	if [ "$option1" = "-d" ] || [ "$option2" = "-d" ]
	then echo $curl_output
	fi
}

displayOptions(){
	# show possible options and parameter
	echo ""
	echo "Possible parameters:"
	echo "--------------------"
	echo "-b : Returns total bytes received and sent"
	echo "-br: Returns total bytes received"
	echo "-bs: Returns total bytes sent"
	echo "-p : Returns total packets received and sent"
	echo "-pr: Returns total packets received"
	echo "-ps: Returns total packets sent"
	echo "-sp: Returns TCP port for control via https connection"
	echo ""
}

invalidParameterException(){
	echo ""
	echo "Given parameter is invalid!"
	echo ""
	displayOptions
}

dummy_curl_req(){
	service="urn:schemas-upnp-org:service:WANCommonInterfaceConfig:1"
	service="urn:schemas-upnp-org:service:WANCommonInterfaceConfig:1"
        action="GetTotalBytesReceived"
	action="GetAddonInfos"
        control_url="/igdupnp/control/WANCommonIFC1"
	control_url="/igdupnp/control/WANCommonIFC1"
        create_xml
        curl_request
        totalBytesReceived=$(echo $curl_output | awk -F"<NewTotalBytesReceived>" '{print $2}' | awk -F"</NewTotalBytesReceived>" '{print $1}')
}

show_total_bytes_sent(){
	service="urn:schemas-upnp-org:service:WANCommonInterfaceConfig:1"
	action="GetTotalBytesSent"
	control_url="/igdupnp/control/WANCommonIFC1"
	create_xml
	curl_request
	totalBytesSent=$(echo $curl_output | awk -F"<NewTotalBytesSent>" '{print $2}' | awk -F"</NewTotalBytesSent>" '{print $1}')
	echo "Total bytes sent:         "$totalBytesSent
}

show_total_bytes_received(){
        service="urn:schemas-upnp-org:service:WANCommonInterfaceConfig:1"
        action="GetTotalBytesReceived"
        control_url="/igdupnp/control/WANCommonIFC1"
        create_xml
        curl_request
        totalBytesReceived=$(echo $curl_output | awk -F"<NewTotalBytesReceived>" '{print $2}' | awk -F"</NewTotalBytesReceived>" '{print $1}')
        echo "Total bytes received:     "$totalBytesReceived
}

show_total_packets_received(){
        service="urn:schemas-upnp-org:service:WANCommonInterfaceConfig:1"
        action="GetTotalPacketsReceived"
        control_url="/igdupnp/control/WANCommonIFC1"
        create_xml
        curl_request;
        totalPacketsReceived=$(echo $curl_output | awk -F"<NewTotalPacketsReceived>" '{print $2}' | awk -F"</NewTotalPacketsReceived>" '{print $1}')
        echo "Total packets received:   "$totalPacketsReceived
}

show_total_packets_sent(){
        service="urn:schemas-upnp-org:service:WANCommonInterfaceConfig:1"
        action="GetTotalPacketsSent"
        control_url="/igdupnp/control/WANCommonIFC1"
        create_xml
        curl_request
        totalPacketsSent=$(echo $curl_output | awk -F"<NewTotalPacketsSent>" '{print $2}' | awk -F"</NewTotalPacketsSent>" '{print $1}')
        echo "Total packets sent:       "$totalPacketsSent
}

show_total_bytes_stats(){
	show_total_bytes_received
	show_total_bytes_sent
}

show_total_packets_stats(){
	show_total_packets_received
	show_total_packets_sent
}

show_wan_stats(){
	show_total_packets_stats
	show_total_bytes_stats
}

check_box_url
dummy_curl_req

if [ $# -eq 0 ]
then show_wan_stats
else
	case "$option1" in
		"-bs")
			show_total_bytes_sent ;;
		"-br")
			show_total_bytes_received ;;
		"-pr")
			show_total_packets_received ;;
		"-ps")
			show_total_packets_sent ;;
		"-p")
			show_total_packets_stats ;;
		"-b")
			show_total_bytes_stats ;;
		"-h")
			displayOptions ;;
		"-a")
			show_total_bytes_received ;;
		"-d")
			show_wan_stats ;;
		*)
			invalidParameterException ;;
	esac
fi
