#!/bin/ash

# get the necessary input from other parts of the system
CWD=$(dirname $0)

while getopts r:p:du opt
do
	case $opt in
		r) ROUTERIP=$OPTARG;;
		p) PASSWORD=$OPTARG;;
		d) OPTION="-d";;
		u) OPTION="-u";;
	esac
done

# these are specific files that need to be found
LOGIN_HTML="html/login/index.html"
LOGIN_JSON="data/Login.json"
INTERFACE_JSON="data/interfaces.json"
DSL_JSON="data/dsl.json"
STATUS_JSON="data/Status.json"

# exit status
BUSY=1
IDLE=0

# encrypted password
encrypt_md5(){
	encrypted_pw=$(echo -n "$PASSWORD" | md5sum | awk -F" " '{print $1}')
	#echo $encrypted_pw
}

# router login
login(){
	if [ -z "$PASSWORD" ]; then
		echo "Password is missing."
		exit 1
	fi
	FIRMWARE_VERSION=$(ash "$CWD/../speedport/identify_model.sh" -f)
	version=$(echo $FIRMWARE_VERSION | awk -F"." '{print $1}')
	if [ "$version" == "05011602" ]
	then
		login_old
	else
		login_new
	fi
}

login_old(){
	encrypt_md5
	login_url="http://$ROUTERIP/$LOGIN_JSON"
	login_out=$(curl -s -m 5 -c "$CWD/cookie.txt" $login_url -H "Content-Type: application/x-www-form-urlencoded" -d "csrf_token=nulltoken&password=$encrypted_pw&showpw=0")
	logged_in=$(echo $login_out | awk -F'"vartype":"status", "varid":"login", "varvalue":"' '{print $2}' | awk -F'"' '{print $1}')
	if [ "$logged_in" == "failed" ]
	then
		echo "Login failed!"
		exit 1
	fi
}

# random number from login page
get_challenge(){
        CHALLENGE=$(curl -s -m 5 "http://$ROUTERIP/$LOGIN_HTML" | grep "challenge" | awk -F'"' '{print $2}')
}

# encrypted password
encrypt_sha256(){
        PASSWORD_SHA256=$(echo -n "$CHALLENGE:$PASSWORD" | sha256sum | awk -F" " '{print $1}')
}

# router login
login_new(){
	get_challenge
	encrypt_sha256
        login_out=$(curl -s -m 5 -c "$CWD/cookie.txt" "http://$ROUTERIP/$LOGIN_JSON" -H "Content-Type: application/x-www-form-urlencoded" -d "csrf_token=nulltoken&password=$PASSWORD_SHA256&showpw=0&challengev=$CHALLENGE")
        logged_in=$(echo $login_out | awk -F'"vartype":"status", "varid":"login", "varvalue":"' '{print $2}' | awk -F'"' '{print $1}')
        if [ "$logged_in" == "failed" ]
        then
                echo "Login failed!"
                exit 1
        fi
}

# link_layer page extraction
interface_page(){
	interface_url="http://$ROUTERIP/$INTERFACE_JSON"
	#echo $interface_url
	page_source=$(curl -s -m 5 -H "Content-Type: application/json, text/javascript, */*; Accept-Encoding: gzip, deflate" -b "$CWD/cookie.txt" $interface_url)
	#echo $page_source
	lan1_rx=$(echo $page_source | awk -F"'rx_packets':'" '{print $2}' | awk -F"'" '{print $1}')
	lan2_rx=$(echo $page_source | awk -F"'rx_packets':'" '{print $3}' | awk -F"'" '{print $1}')
	lan3_rx=$(echo $page_source | awk -F"'rx_packets':'" '{print $4}' | awk -F"'" '{print $1}')
	lan4_rx=$(echo $page_source | awk -F"'rx_packets':'" '{print $5}' | awk -F"'" '{print $1}')
	wl24_rx=$(echo $page_source | awk -F"'rx_packets':'" '{print $6}' | awk -F"'" '{print $1}')
	wl5_rx=$(echo $page_source | awk -F"'rx_packets':'" '{print $7}' | awk -F"'" '{print $1}')
	dhcp_rx=$(echo $page_source | awk -F"'rx_packets':'" '{print $8}' | awk -F"'" '{print $1}')
	ppp_rx=$(echo $page_source | awk -F"'rx_packets':'" '{print $9}' | awk -F"'" '{print $1}')
	lan1_tx=$(echo $page_source | awk -F"'tx_packets':'" '{print $2}' | awk -F"'" '{print $1}')
	lan2_tx=$(echo $page_source | awk -F"'tx_packets':'" '{print $3}' | awk -F"'" '{print $1}')
	lan3_tx=$(echo $page_source | awk -F"'tx_packets':'" '{print $4}' | awk -F"'" '{print $1}')
	lan4_tx=$(echo $page_source | awk -F"'tx_packets':'" '{print $5}' | awk -F"'" '{print $1}')
	wl24_tx=$(echo $page_source | awk -F"'tx_packets':'" '{print $6}' | awk -F"'" '{print $1}')
	wl5_tx=$(echo $page_source | awk -F"'tx_packets':'" '{print $7}' | awk -F"'" '{print $1}')
	dhcp_tx=$(echo $page_source | awk -F"'tx_packets':'" '{print $8}' | awk -F"'" '{print $1}')
	ppp_tx=$(echo $page_source | awk -F"'tx_packets':'" '{print $9}' | awk -F"'" '{print $1}')
	#echo "LAN1 In_Frames: $lan1_rx"
	#echo "LAN2 In_Frames: $lan2_rx"
	#echo "LAN3 In_Frames: $lan3_rx"
	#echo "LAN4 In_Frames: $lan4_rx"
	#echo "WLAN 2.4 GHz In_Frames: $wl24_rx"
	#echo "WLAN 5 GHz In_Frames: $wl5_rx"
	#echo "DHCP In_Frames: $dhcp_rx"
	#echo "PPP In_Frames: $ppp_rx"
	#echo "LAN1 Out_Frames: $lan1_tx"
	#echo "LAN2 Out_Frames: $lan2_tx"
	#echo "LAN3 Out_Frames: $lan3_tx"
	#echo "LAN4 Out_Frames: $lan4_tx"
	#echo "WLAN 2.4 GHz Out_Frames: $wl24_tx"
	#echo "WLAN 5 GHz Out_Frames: $wl5_tx"
	#echo "DHCP Out_Frames: $dhcp_tx"
	#echo "PPP Out_Frames: $ppp_tx"
}

# check if network is busy
calc_network_state(){
	interface_page
	out1=$ppp_tx
	in1=$ppp_rx
	through1=$(( out1+in1 ))
	# it seems the easiest implementation is to wait for exactly one second in between measurements. Other waiting times may need additional packages.
	sleep $SLEEPTIME
	interface_page
	out2=$ppp_tx
	in2=$ppp_rx
	through2=$(( out2+in2 ))
	through=$(( through2-through1 ))
	clean_cookies
	if [ "$through" -gt "$THRESHOLD" ]
	then
		exit $BUSY
	else
        	exit $IDLE
	fi
}

# fetch connection details
fetch_conn_details(){
	dsl_url="http://$ROUTERIP/$DSL_JSON"
	page_source=$(curl -s -m 5 -H "Content-Type: application/json, text/javascript, */*; Accept-Encoding: gzip, deflate" -b "$CWD/cookie.txt" $dsl_url)
	dl_rate=$(echo $page_source | awk -F"'dactual' :'" '{print $2}' | awk -F"'" '{print $1}')
	ul_rate=$(echo $page_source | awk -F"'uactual' :'" '{print $2}' | awk -F"'" '{print $1}')
	echo "Actual download rate: $dl_rate"
	echo "Actual upload rate:   $ul_rate"
}


clean_cookies(){
	rm "$CWD/cookie.txt"
}


# MAIN
login
interface_page
if [ "$OPTION" == "-c" ]; then
	fetch_conn_details
elif [ "$OPTION" == "-d" ]; then
	echo $ppp_rx
elif [ "$OPTION" == "-u" ]; then
	echo $ppp_tx
else
	echo "Option parameter is missing."
fi

clean_cookies

exit 0
