#!/bin/sh

CWD=$(dirname $0)
ROUTER_IP=$(/bin/sh $CWD/../general/get_router_ip.sh)

while getopts r:tp:duc: opt
do
	case $opt in
                p) PASSWORD=$OPTARG;;
		t) OPTION="-t";;
		d) OPTION="download";;
		u) OPTION="upload";;
		c)
			OPTION="channel"
			NEW_CHANNEL=$OPTARG;;
	esac
done

create_xml(){
	xml_data="<?xml version='1.0' encoding='utf-8'?><s:Envelope s:encodingStyle='http://schemas.xmlsoap.org/soap/encoding/' xmlns:s='http://schemas.xmlsoap.org/soap/envelope/'><s:Body><u:$action xmlns:u='$service'></u:$action></s:Body></s:Envelope>"
}

create_xml_change_channel(){
	xml_data="<?xml version='1.0' encoding='utf-8'?><s:Envelope s:encodingStyle='http://schemas.xmlsoap.org/soap/encoding/' xmlns:s='http://schemas.xmlsoap.org/soap/envelope/'><s:Body><u:$action xmlns:u='$service'><$argument>$NEW_CHANNEL</$argument></u:$action></s:Body></s:Envelope>"
}

curl_request(){
	curl_output=$(curl -s -m 5 http://$ROUTER_IP:49000$control_url -H "Content-Type: text/xml; charset='utf-8'" -H "SoapAction:$service#$action" -d "$xml_data")
}

curl_request_https(){
	curl_output=$(curl -s -k -m 5 --anyauth --user dslf-config:$PASSWORD https://$ROUTER_IP:$PORT$control_url -H "Content-Type: text/xml; charset='utf-8'" -H "SoapAction:$service#$action" -d "$xml_data")
}


change_channel(){
	service="urn:dslforum-org:service:WLANConfiguration:1"
	control_url="/upnp/control/wlanconfig1"
	action="SetChannel"
	argument="NewChannel"
	create_xml_change_channel
	curl_request_https
}


service="urn:dslforum-org:service:DeviceInfo:1"
control_url="/upnp/control/deviceinfo"
action="GetSecurityPort"

create_xml
curl_request
PORT=$(echo $curl_output | awk -F'<NewSecurityPort>' '{print $2}' | awk -F'</NewSecurityPort>' '{print $1}')

if [ "$OPTION" = "-t" ]; then
	echo $PORT
	exit 0
elif [ -z "$PASSWORD" ]; then
	echo "Router password parameter is missing."
	exit 1
else

service="urn:dslforum-org:service:WANCommonInterfaceConfig:1"
control_url="/upnp/control/wancommonifconfig1"
action="GetTotalBytesSent"

create_xml
curl_request_https
totalBytesSent=$(echo $curl_output | awk -F"<NewTotalBytesSent>" '{print $2}' | awk -F"</NewTotalBytesSent>" '{print $1}')

action="GetTotalBytesReceived"
create_xml
curl_request_https
totalBytesReceived=$(echo $curl_output | awk -F"<NewTotalBytesReceived>" '{print $2}' | awk -F"</NewTotalBytesReceived>" '{print $1}')
fi

if [ "$OPTION" == "upload" ]; then
	echo "$totalBytesSent"
elif [ "$OPTION" == "download" ]; then
	echo "$totalBytesReceived"
elif [ "$OPTION" == "channel" ]; then
	if [ -z "$2" ]; then
		echo "Channel parameter is missing."
		exit 1
	fi
	change_channel
else
	echo "Total Bytes Sent:     $totalBytesSent"
	echo "Total Bytes Received: $totalBytesReceived"
fi

exit 0
