#!/bin/sh

source ./dynamic.conf
source ./static.conf

RESP_TOKEN=$(/usr/bin/curl -s "$TOKEN_URL" \
        -H 'Content-Type: application/json' \
        --data-binary '{"email":"'$FB_EMAIL'","password":"'$FB_PASSWORD'","returnSecureToken":true}')
ID_TOKEN=$(echo $RESP_TOKEN | /usr/bin/jsonfilter -e '$.idToken')

for i in `seq 1 9`; do
        ubus call iwinfo scan '{"device":"radio0"}' > /dev/null
done

timestamp=$(date +%s)
day="$(($timestamp/86400))"
sec="$(($timestamp-$day*86400))"
timestamp=$(($timestamp*1000))

base_out=$(ubus call iwinfo scan '{"device":"radio0"}' | jsonfilter -e '$.results')
bssids=$(jsonfilter -s "$base_out" -e '@.*.bssid')
n_entry=$(echo $bssids | awk '{print NF}')

get_bssid() {
    local inp=$1
    local bssid
    bssid=$(jsonfilter -s "$inp" -e '@.bssid')
    echo "$bssid"
}

get_capabilities() {
    local inp=$1
    local base_out=$(jsonfilter -s "$inp" -e '@.encryption')
    local is_enc=$(jsonfilter -s "$base_out" -e '@.enabled')
    local capa
    if [ "$is_enc" == "true" ]; then
        local wpa=$(jsonfilter -s "$base_out" -e '@.wpa.*' | tr 'a-z' 'A-Z')
        local wep=$(jsonfilter -s "$base_out" -e '@.wep.*' | tr 'a-z' 'A-Z')
        if [ -n "$wpa" ]; then
            local auth=$(jsonfilter -s "$base_out" -e '@.authentication.*' | tr 'a-z' 'A-Z')
            local ciph=$(jsonfilter -s "$base_out" -e '@.ciphers.*' | tr 'a-z' 'A-Z' | tr "\n" "-" | sed 's/.$//')
            n_wpa=$(echo $wpa | wc -w)
            if [ "$n_wpa" -eq 2 ]; then
                wpa="WPA$(echo $wpa | awk '{print $1}') WPA$(echo $wpa | awk '{print $2}')"
            elif [ "$n_wpa" -eq 1 ]; then
                wpa="WPA$wpa"
            fi
            capa=$(echo "[$wpa $auth $ciph]" | tr " " "-")
        elif [ -n "$wep" ]; then
            capa="[WEP]"
        fi
    else
        capa="[]"
    fi
    echo "$capa"
}

get_freq() {
    local inp=$1
    local freq
    chan=$(jsonfilter -s "$inp" -e '@.channel')
    freq=$(($chan*5+2407))
    echo "$freq"
}

get_level() {
    local inp=$1
    local level
    level=$(jsonfilter -s "$inp" -e '@.signal')
    echo "$level"
}

get_ssid() {
    local inp=$1
    local ssid
    ssid=$(jsonfilter -s "$inp" -e '@.ssid')
    echo "$ssid"
}

get_weight() {                                                              
    local inp=$1                                                                                     
    local weight                                                                            
    if [ "$inp" -ge -30 ]; then                                                      
        weight=100                                                          
    fi                                                                                               
    if [ "$inp" -gt -66 ] && [ "$inp" -le -29 ]; then                       
        weight=54                                                                           
    fi                                                                               
    if [ "$inp" -eq -66 ]; then                                                             
        weight=48                                                                    
    fi                                                                      
    if [ "$inp" -ge -71 ] && [ "$inp" -lt -66 ]; then                                       
        weight=36                                                                    
    fi                                                                                      
    if [ "$inp" -ge -75 ] && [ "$inp" -le -72 ]; then                                
        weight=24                                                                           
    fi                                                                                      
    if [ "$inp" -gt -78 ] && [ "$inp" -le -76 ]; then                                
        weight=18                                                           
    fi                                                                      
    if [ "$inp" -eq -78 ] && [ "$inp" -eq -79 ]; then       
        weight=12                                                                           
    fi                                                                               
    if [ "$inp" -ge -82 ] && [ "$inp" -lt -79 ]; then                       
        weight=9                                                            
    fi                                                      
    if [ "$inp" -gt -91 ] && [ "$inp" -le -83 ]; then                                       
        weight=6                                                                     
    fi                                                                      
    if [ "$inp" -lt -90 ]; then                                             
        weight=1                                     
    fi                                                                                      
    weight=$(jq -n "$weight/100")                                                    
    echo $weight                                                            
}

get_all_prop() {
    local inp=$1
    bssid=$(get_bssid "$inp")
    capa=$(get_capabilities "$inp")
    freq=$(get_freq "$inp")
    level=$(get_level "$inp")
    ssid=$(get_ssid "$inp")
    weight=$(get_weight "$level")
}

print_res() {
    echo "BSSID: $bssid"
    echo "Capabilities: $capa"
    echo "Frequency: $freq"
    echo "Level: $level"
    echo "SSID: $ssid"
    echo "Timestamp: $timestamp"
    echo "Weight: $weight"
    echo ""
}

data="{"
my_ssid=""
my_bssid=""
my_level=-100
for i in `seq 0 $(expr $n_entry - 1)`; do
    entry=$(jsonfilter -s "$base_out" -e "@[$i]")
    get_all_prop "$entry"
    # print_res
    if [ "$level" -gt "$my_level" ]; then
        my_level=$level
        my_ssid=$ssid
        my_bssid=$bssid
    fi
    tmp_data='"'"$bssid"'": {"SSID": "'"$ssid"'", "capabilities": "'"$capa"'", "frequency": "'"$freq"'", "level": "'"$level"'", "timestamp": "'"$timestamp"'", "weight": "'"$weight"'"}'
    if [ "$i" -eq 0 ]; then
        data="${data}${tmp_data}"
    else
        data="${data}, ${tmp_data}"
    fi
done

# Upload charlie wifi scan to /charlies/<cid>/wifiScan
data="${data}}"
#echo $data
statuscode=$(/usr/bin/curl -s -w "%{http_code}" -o /dev/null -X PUT -d "$data" $FIREBASE_URL/charlies/$CID/wifiScan/$day/$sec/.json?auth=$ID_TOKEN)
#echo $statuscode

ap_data='{"apSsid": "'"$my_ssid"'", "apBssid": "'"$my_bssid"'"}'

# Upload AP's information (SSID, BSSID) to users/<uid>/charlies/<cid>
statuscode2=$(/usr/bin/curl -s -w "%{http_code}" -o /dev/null -X PATCH -d "$ap_data" $FIREBASE_URL/users/$UID/charlie/charlies/$CID/.json?auth=$ID_TOKEN)
#echo $statuscode2

# Upload AP's information (SSID, BSSID) to /charlies/<cid>/
statuscode3=$(/usr/bin/curl -s -w "%{http_code}" -o /dev/null -X PATCH -d "$ap_data" $FIREBASE_URL/charlies/$CID/.json?auth=$ID_TOKEN)
#echo $statuscode3
